/* -*-c-*-
 *
 * $Id: dh.h,v 1.3 2000/02/12 18:21:02 mdw Exp $
 *
 * Diffie-Hellman and related public-key systems
 *
 * (c) 1999 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: dh.h,v $
 * Revision 1.3  2000/02/12 18:21:02  mdw
 * Overhaul of key management (again).
 *
 */

#ifndef CATACOMB_DH_H
#define CATACOMB_DH_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/

#ifndef CATACOMB_GRAND_H
#  include "grand.h"
#endif

#ifndef CATACOMB_PGEN_H
#  include "pgen.h"
#endif

/*----- Data structures ---------------------------------------------------*/

typedef struct dh_param {
  mp *p, *q;
  mp *g;
} dh_param;

/*----- Functions provided ------------------------------------------------*/

/* --- @dh_gen@ --- *
 *
 * Arguments:	@dh_param *dp@ = pointer to output parameter block
 *		@unsigned ql@ = length of %$q$% in bits, or zero
 *		@unsigned pl@ = length of %$p$% in bits
 *		@unsigned steps@ = number of steps to go
 *		@grand *r@ = random number source
 *		@pgen_proc *event@ = event handler function
 *		@void *ectx@ = argument for the event handler
 *
 * Returns:	@PGEN_DONE@ if it worked, @PGEN_ABORT@ if it didn't.
 *
 * Use:		Generates Diffie-Hellman parameters.
 *
 *		The parameters are a prime %$q$%, relatively small, and a
 *		large prime %$p = kq + 1$% for some %$k$%, together with a
 *		generator %$g$% of the cyclic subgroup of order %$q$%.  These
 *		are actually the same as the DSA parameter set, but the
 *		generation algorithm is different.  Also, if @ql@ is zero,
 *		this algorithm forces %$k = 2$%, and chooses %$g = 4$%.  Make
 *		sure you have something interesting to do if you choose this
 *		option.
 */

extern int dh_gen(dh_param */*dp*/, unsigned /*ql*/, unsigned /*pl*/,
		  unsigned /*steps*/, grand */*r*/, pgen_proc */*event*/,
		  void */*ectx*/);

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
